# Trustee KeyProvider (TKP) implementation of Hadoop Key Provider #

### Install Hadoop with KMS from source ###

* Documentation for the KMS can be found in the KMS source module in the 
  src/main/site directory in the [index.apt.vm](https://github.com/apache/hadoop/blob/trunk/hadoop-common-project/hadoop-kms/src/site/apt/index.apt.vm) file
  
Build Apache Hadoop 3.0.0-SNAPSHOT from Apache
----------------------------------------------

  ***IMPORTANT UPDATE: September 17, 2014***
  
    $ git clone git://git.apache.org/hadoop.git 
    $ git checkout trunk 
    $ mvn clean package -Pdist -Dtar -DskipTests -Dmaven.javadoc.skip=true
   
The resulting Hadoop tarball will be in hadoop-dist/target
    
NOTE: If you've never built Hadoop on your machine before,
you'll need to build and install ProtocolBuffers before building Hadoop.
See http://wiki.apache.org/hadoop/HowToContribute#Building_ProtocolBuffers_.28for_0.23.2B-.29
(Linux instructions will also work on Mac)

If trouble building on MacOS, see here: 
https://issues.apache.org/jira/browse/HADOOP-9350
https://ask.opendaylight.org/question/329/jdk-error-during-maven-build/

Start KMS with (default) JavaKeyStoreProvider
---------------------------------------------
    
    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/sbin
    $ ./kms.sh start
    

KMS config files are in hadoop-3.0.0-SNAPSHOT/etc/hadoop (with others)
Log files are in hadoop-3.0.0-SNAPSHOT/logs (see esp. kms.log)

Send test commands to KMS
-------------------------
    
    $ curl http://localhost:16000/kms/v1/keys/names?user.name=anthony
    [ ]
        
NOTE: Username can be anything since simple authentication is configured
       
    $ vi key.json
    {
    "name"        : "ajy-key",
    "cipher"      : "AES/CTR/NoPadding",
    "length"      : 256,        
    "description" : "Anthony's test key"
    }
    
    $ curl -H "Content-Type: application/json" --data @key.json http://localhost:16000/kms/v1/keys?user.name=anthony
    {
    "versionName" : "ajy-key@0",
    "material" : "n/J+3omy7uu42EVZbW9s8qfOQTHBK6Jm2jW90WVFnsY="
    }
    
NOTE: Find more methods to test in the docs noted in the first bullet.

Stop KMS
--------
    
    $ ./kms.sh stop
    
### Build, install and configure KeyTrustee key provider ###    

Build ztrusteekp JAR
--------------------
   
    mvn package
      
If you are using the Gazzang repo and example settings.xml file, be sure to specify the dev profile:
   
    mvn -Pdev package
   
   
Copy the keytrusteekp JAR to KMS Tomcat web-app WEB-INF/lib/ directory
----------------------------------------------------------------------
    
    cp target/keytrusteekp-1.0.0-cdh5.2.0-SNAPSHOT.jar ~/repos/hadoop/hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/share/hadoop/kms/tomcat/webapps/kms/WEB-INF/lib/
    
    
Copy keytrustee key provider dependencies to KMS Tomcat web-app WEB-INF/lib/ directory
--------------------------------------------------------------------------------------
    
    cp -n target/lib/*  ~/repos/hadoop/hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/share/hadoop/kms/tomcat/webapps/kms/WEB-INF/lib/
        
NOTE: -n argument prevents cp command from overwriting existing files in lib directory

    bcpg-jdk14-1.47.jar
    bcprov-ext-jdk14-1.47.jar
    bcprov-jdk14-1.47.jar
    commons-codec-1.4.jar 
    commons-compress-1.4.1.jar
    commons-io-2.4.jar 
    commons-lang-2.6.jar
    jcommander-1.27.jar
    json-20070829.jar
    log4j-1.2.17.jar 
    maven-metadata-appassembler.xml
    slf4j-api-1.7.5.jar 
    slf4j-log4j12-1.7.5.jar 
    xz-1.0.jar

    guava-11.0.2.jar
    gson-2.2.4.jar
    jsr305-1.3.9.jar
    commons-pool2-2.2.jar

Start keytrustee server on a local or remote machine and create and org/auth, if necessary.
------------------------------------------------------------------------------------------

See keytrustee server README for more details: 
https://bitbucket.org/eng-gazzang/keytrustee

Download keytrustee server certificate to KMS machine
-----------------------------------------------------
    
    echo -n | openssl s_client -connect ayoungga-keytrustee.vpc.cloudera.com:443 | sed -ne '/-BEGIN CERTIFICATE-/,/-END CERTIFICATE-/p' > /tmp/keytrustee.pem 

The server certificate is needed to establish that the client trusts the recognizes the server as a valid endpoint for secure communication.


Add the keytrustee server certificate to your JVM trust store
--------------------------------------------------------------  

    keytool -import -trustcacerts -alias ayoungga-keytrustee.vpc.cloudera.com -file /tmp/keytrustee.pem -keystore /Library/Java/JavaVirtualMachines/jdk1.7.0_45.jdk/Contents/Home/jre/lib/security/jssecacerts
    
OR

    keytool -import -trustcacerts -alias ayoungga-keytrustee.vpc.cloudera.com -file /tmp/keytrustee.pem -keystore /etc/mycerts
    -Djavax.net.ssl.trustStore=/etc/mycerts  (use this system property when starting the JVM)
        
NOTE: Another common location on mac for the trust store is /System/Library/Frameworks/JavaVM.framework/Versions/CurrentJDK/Home/lib/security
NOTE2: To be sure of your JVM filesystem location, look at the output messages when running sbin/kms.sh run

~~Register KeyTrustee client on KMS machine~~
--------------------------------------------- 

   NOTE: This step is no longer required as self-registration is functional. However you
   can still perform this step if you don't want to specify the properties in the kms-site.xml
   file as described two steps down.
   
     
     ./bin/ztrustee-java register -c /tmp/keytrustee.pem -s <SERVER_HOSTNAME> --org <ORG_NAME>
     
     
Configure the kms-site.xml with the KeyTrustee KeyProvider URI
-------------------------------------------

    etc/hadoop/kms-site.xml

      <property>
        <name>hadoop.kms.key.provider.uri</name>
        <value>keytrustee://file@/${user.home}/.keytrustee/</value>
        <description/>
      </property>

    (comment out any existing key provider OR create a comma-separated list of path values)

~~Configure the core-site.xml with the KeyTrustee KeyProvider URI~~
------------------------------------------
(NO LONGER REQUIRED AS OF CDH 5.3.0)

    etc/hadoop/core-site.xml

      <property>
        <name>hadoop.security.key.provider.path</name>
        <value>keytrustee://file@/${user.home}/.keytrustee/</value>
        <description/>
      </property>

    (comment out any existing key provider OR create a comma-separated list of path values)
    
Create and configure the kts-site.xml file with KeyTrustee key provider properties 
------------------------------
      
      <!-- KeyTrustee Key Provider properties -->
      <property>
          <name>cloudera.trustee.keyprovider.hostname-ACTIVE</name>
          <value>SERVER_HOSTNAME</value>
      </property>

      <property>
          <!-- this property is optional but required for HA -->
          <name>cloudera.trustee.keyprovider.hostname-PASSIVE</name>
          <value>SERVER2_HOSTNAME</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.org</name>
          <value>ORG_NAME</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.auth</name>
          <value>AUTH_CODE</value>
      </property>

      <property>
          <!-- This property is optional but is required until the trust store
          instructions above are complete. After configuring keystore, set to false. -->
          <name>cloudera.trustee.keyprovider.insecure</name>
          <value>true</value>
      </property>

      <property>
           <!-- New in CDH 5.4.0 -->
           <name>cloudera.trustee.keyprovider.roundrobin</name>
           </value>false</value>
      </property>

      
      <property>
          <name>cloudera.trustee.keyprovider.keylength</name>
          <value>4096</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.protocol</name>
          <value>json-encrypt</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.tokensync</name>
          <value>false</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.usepool</name>
          <value>true</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.stronglocking</name>
          <value>false</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.pool.max</name>
          <value>5</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.pool.min.idle</name>
          <value>1</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.pool.max.idle</name>
          <value>2</value>
      </property>

      <property>
          <name>cloudera.trustee.keyprovider.pool.abandoned.timeout</name>
          <value>300</value>
      </property>
      
      
Start KMS
---------
    
        sbin/kms.sh run
    
      
Configure Hadoop core-site.xml with the KMS KeyProvider URI.
------------------------------------------------------------

    etc/hadoop/core-site.xml

    (no changes needed currently)

Use the KeyTool to access the KMS
---------------------------------

    $ ./bin/hadoop key ...
    

    $ ./bin/hadoop key create ajy-testkey-1 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key create ajy-testkey-2 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key list -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key roll ajy-testkey-2 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key delete ajy-testkey-2 -provider kms://http@localhost:16000/kms

Configuring High Availability
-----------------------------

If you configured two keytrustee servers (active and passive) in the kts-site.xml file as described four sections above,
then no further action is required on the client to configure high availability. 

However, if you did not configure two servers during the initial setup and before the key provider registered with the
KeyTrustee server, then in addition to adding the second server to the kts-site.xml file above you must
also update an internal file as described below.

Again, the steps below are only required if you did not initially configure two servers in the kts-site.xml file before
using the key provider for the first time.

#### Add a passive server entry to the kts-site.xml file as described four sections above. ####

      <!-- KeyTrustee Key Provider properties -->
      <property>
          <name>cloudera.trustee.keyprovider.hostname-ACTIVE</name>
          <value>SERVER_HOSTNAME</value>
      </property>

      <property>
          <!-- this property is optional but required for HA -->
          <name>cloudera.trustee.keyprovider.hostname-PASSIVE</name>
          <value>SERVER2_HOSTNAME</value>
      </property>

#### Determine the location of the internal keytrustee client registration directory by looking in the kms-site.xml file. ####
 
      <!-- KMS Backend KeyProvider -->
      <property>
          <name>hadoop.security.key.provider.path</name>
          <value></value>
          <value>keytrustee://file@/${user.home}/.keytrustee/</value> 
          <description/>
      </property>
 
For example, in the kms-site.xml file snippet above, the keytrustee client registration 
directory is at ${user.home}/.keytrustee 
 
#### Edit the keytrustee client registration profile *keytrustee.conf* in the keytrustee client registration directory. ####

    {
    "FAILOVER": "True",
    "REMOTES": {"ayoungga-keytrustee3.vpc.cloudera.com": {
        "TOKEN_SYNC": false,
        "SSL_INSECURE": true,
        "REMOTE_SERVER": "https://ayoungga-keytrustee3.vpc.cloudera.com",
        "PROTOCOL": "json-encrypt",
        "REMOTE_FINGERPRINT": "4096R/5E96CF9D1615A5A04665DB4C699B34B724B7A2F0",
        "REMOTE_SERVERS": [
            "ayoungga-keytrustee3.vpc.cloudera.com",
            "ayoungga-keytrustee4.vpc.cloudera.com"
        ]
    }},
    "LOCAL_FINGERPRINT": "4096R/D333EDF96117461B764AB3C780B2492C13182824"
    }

Add a second server to the REMOTE_SERVERS array in the configuration file (ayoungga-keytrustee4.vpc.cloudera.com in 
the example above). Do not change the order of the servers. The first server listed in the REMOTE_SERVERS array must
match the primary server listed higher in the configuration file (under REMOTE_SERVER as opposed to REMOTE_SERVERS).


Undeleting and purging KeyTrustee Key Provider methods via the REST interface
-----------------------------------------

The delete operation on the Trustee key provider is a "soft delete", meaning that is possible to "undelete" the key.
It is also possible to "purge" the key to delete it permanently. Because these operations are not part of the standard
Hadoop key provider API, they are not currently exposed through Hadoop KeyShell (hadoop key). However, you can call these
operations directly via the Trustee key provider REST API.

See the examples below.

#### Use KeyShell to list existing keys

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ ./bin/hadoop key list -provider kms://http@localhost:16000/kms
    
    Listing keys for KeyProvider: KMSClientProvider[http://localhost:16000/kms/v1/]
    ajy-testkey-1

#### Use KeyShell to delete an existing key

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ ./bin/hadoop key delete ajy-testkey-1 -provider kms://http@localhost:16000/kms

    Deleting key: ajy-testkey-1 from KeyProvider: KMSClientProvider[http://localhost:16000/kms/v1/]
    ajy-testkey-1 has been successfully deleted.
    KMSClientProvider[http://localhost:16000/kms/v1/] has been updated.
    
#### Use KeyShell to verify the key was deleted

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ ./bin/hadoop key list -provider kms://http@localhost:16000/kms

    Listing keys for KeyProvider: KMSClientProvider[http://localhost:16000/kms/v1/]
    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$

#### Use the KeyTrustee key provider REST API to undelete the deleted key

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ curl -L -d "trusteeOp=undelete" "http://localhost:16000/kms/v1/trustee/key/ajy-testkey-1?user.name=anthony&trusteeOp=undelete"

#### Use KeyShell to verify the key was restored

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ ./bin/hadoop key list -provider kms://http@localhost:16000/kms

    Listing keys for KeyProvider: KMSClientProvider[http://localhost:16000/kms/v1/]
    ajy-testkey-1

#### Use the KeyTrustee key provider REST API to purge the restored key

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ curl L -d "trusteeOp=purge" "http://localhost:16000/kms/v1/trustee/key/ajy-testkey-1?user.name=anthony&trusteeOp=purge"

#### Use KeyShell to verify the key was deleted

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ ./bin/hadoop key list -provider kms://http@localhost:16000/kms

    Listing keys for KeyProvider: KMSClientProvider[http://localhost:16000/kms/v1/]
    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ 

#### Use the KeyTrustee key provider REST API to attempt to undelete the purged key

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ curl -L -d "trusteeOp=undelete" "http://localhost:16000/kms/v1/trustee/key/ajy-testkey-1?user.name=anthony&trusteeOp=undelete"
    {
      "RemoteException" : {
        "message" : "Key with name ajy-testkey-1 not found in com.cloudera.keytrustee.TrusteeKeyProvider@6786dc7d",
        "exception" : "IOException",
        "javaClassName" : "java.io.IOException"
      }
    }
    
Migrating keys from JKS key provider to TKP via the REST interface
-----------------------------------------

NEW in CDH 5.3.0

The migration function allows keys to be migrated from the Java Key Store key provider included with Hadoop
to the CDH Key Trustee key provider.

See the examples below.

#### Configure KMS to use Java Key Store key provider

    <property>
      <name>hadoop.kms.key.provider.uri</name>
      <value>jceks://file@/${user.home}/kms.keystore</value>
      <description>
        URI of the backing KeyProvider for the KMS.
      </description>
    </property>

#### Create, roll and delete keys

    $ ./bin/hadoop key create super-key-1 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key create super-key-2 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key create super-key-3 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key roll super-key-2 -provider kms://http@localhost:16000/kms
    $ ./bin/hadoop key delete super-key-3 -provider kms://http@localhost:16000/kms

#### Configure KMS to use Trustee Key Provider (primary) and one or more secondary key providers

    <property>
      <name>hadoop.kms.key.provider.uri</name>
      <value>keytrustee://file@/${user.home}/.keytrustee/,jceks://file@/${user.home}/kms.keystore</value>
      <description>
        URI of the backing KeyProvider for the KMS.
      </description>
    </property>
  
#### Use the KeyTrustee key provider REST API to run the migration operation

    mac-ayoungga-01:hadoop-3.0.0-SNAPSHOT ajyoung$ curl L -d "trusteeOp=migrate" "http://localhost:16000/kms/v1/trustee/key/migrate?user.name=anthony&trusteeOp=migrate"
  
#### Example log output from kms.log

    2014-11-12 23:50:36,488 INFO  TrusteeKeyProvider - Number of existing key versions named super-key-2: 0
    2014-11-12 23:50:43,384 INFO  TrusteeKeyProvider - Migrated key named super-key-2. New version name: QhlonLVnPjOqGz3WlSt5qBfvXUGHaW901pApYiOMpmX. Old version name: super-key-2@0.
    2014-11-12 23:50:45,496 INFO  TrusteeKeyProvider - Number of existing key versions named super-key-2: 1
    2014-11-12 23:50:52,487 INFO  TrusteeKeyProvider - Migrated key named super-key-2. New version name: 7LWoT1BgRWkBSm67ZAGrZg0FUwN7YpCSdn0r7jDJuNM. Old version name: super-key-2@1.
    2014-11-12 23:50:57,712 INFO  TrusteeKeyProvider - Number of existing key versions named super-key-1: 0
    2014-11-12 23:51:05,393 INFO  TrusteeKeyProvider - Migrated key named super-key-1. New version name: R8mc4mOiA4s5Ps7zoKCnZtPDIztGC3VMWKEwJvJpYUS. Old version name: super-key-1@0.

#### Example log output from kms.log when some keys have already been migrated

    2014-11-12 23:51:05,393 INFO  TrusteeKeyProvider - Migrated key named super-key-1. New version name: R8mc4mOiA4s5Ps7zoKCnZtPDIztGC3VMWKEwJvJpYUS. Old version name: super-key-1@0.
    2014-11-12 23:51:09,864 WARN  TrusteeKeyProvider - Found proposed key version name (ajy-key@0) in target provider already. Skipping.
    2014-11-12 23:51:09,864 WARN  TrusteeKeyProvider - Existing key version name: PYfiosqZs8sJfognIggpn1DbLfgysNR3Ca3drmObqi5. Existing original_id: ajy-key@0.
    2014-11-12 23:51:13,954 WARN  TrusteeKeyProvider - Found proposed key version name (ajy-testkey-hmat@0) in target provider already. Skipping.
    2014-11-12 23:51:13,955 WARN  TrusteeKeyProvider - Existing key version name: XmgMNYvY4k0lNL0ONoAl4yzKCRftFW1uGUBtgP7saod. Existing original_id: ajy-testkey-hmat@0.


Configure ACLs for KeyTrustee undelete, purge and migrate operations
--------------------------
ACLs for the KeyTrustee specific undelete, purge and migrate operations are configured in kts-acls.xml. Place this file in the
same location as your kms-acls.xml file. See example below.

    <property>
        <name>keytrustee.kms.acl.UNDELETE</name>
        <value>*</value>
        <description>
            ACL for undelete-key operations.
        </description>
    </property>

    <property>
        <name>keytrustee.kms.acl.PURGE</name>
        <value>*</value>
        <description>
            ACL for purge-key operations.
        </description>
    </property>
    
    <property>
        <name>keytrustee.kms.acl.MIGRATE</name>
        <value>*</value>
        <description>
            ACL for purge-key operations.
        </description>
    </property>
    
Real world test of key migration (JKS KP -> TKP) with HDFS
-----------------------------------------

NEW in CDH 5.3.0

The section above demonstrates simple functional testing of the CDH Key Trustee key provider migration functionality
but a more realistic test should involve not only the key providers themselves, but also the client programs which use
the keys.

The examples below demonstrate how to verify that migrated keys can still be used by external
client (in this case, HDFS).

#### Configure KMS to use Java Key Store key provider

(set the following value in hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/etc/hadoop)

    <property>
      <name>hadoop.kms.key.provider.uri</name>
      <value>jceks://file@/${user.home}/kms.keystore</value>
      <description>
        URI of the backing KeyProvider for the KMS.
      </description>
    </property>

#### Start KMS

    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/
    $ .sbin/kms.sh start
    
#### Start Hadoop mini-cluster 

(preferably in separate shell from KMS process above)

    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/
    $ HADOOP_CLASSPATH=share/hadoop/yarn/test/hadoop-yarn-server-tests-3.0.0-cdh6.0.0-SNAPSHOT-tests.jar bin/hadoop jar \
     ./share/hadoop/mapreduchadoop-mapreduce-client-jobclient-3.0.0-cdh6.0.0-SNAPSHOT-tests.jar minicluster  \
     -writeConfig minidfsconfig.xml -writeDetails minidfsinfo.json -nnport 60908
     
The bin/hadoop command above is all one line. The HADOOP_CLASSPATH variable is required because of [YARN-683](https://issues.apache.org/jira/browse/YARN-683).
For more information about running a Hadoop mini-cluster from a dev source tree, see the following links:
http://hadoop.apache.org/docs/current/hadoop-project-dist/hadoop-common/CLIMiniCluster.html
http://hadoop.apache.org/docs/current/hadoop-project-dist/hadoop-common/SingleCluster.html

You do not have to use port 60908 for your name node port. However, if you do not specify a name node port at all, one will be
randomly chosen and you'll need to look in the minidfsconfig.xml output to find out the active name node port. In any case,
you'll need to know the name node port in later steps.

To stop the mini-cluster, just type CTRL-c in the window where it is running (note that this is different from KMS, 
which when started as shown above, must be stopped using the kms.sh stop command).

#### Update hadoop-dist core-site.xml
In the hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/etc/hadoop directory add to core-site.xml
    
    <property>
      <name>fs.default.name</name>
      <value>hdfs://localhost:60908</value>
    </property>
    <property>
      <name>hadoop.security.key.provider.path</name>
      <value>kms://http@localhost:16000/kms</value>
    </property>
    <property>
      <name>dfs.encryption.key.provider.uri</name>
      <value>kms://http@localhost:16000/kms</value>
    </property>

#### Create a simple core-site.xml and hdfs-site.xml files for use by the HDFS client

In the hadoop-dist/target/hadoop-3.0.0-SNAPSHOT directory, create a directory called mini-conf/
In the hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/mini-conf direcotry, create a file called core-site.xml with 
the following content:

    <configuration>
     <property>
      <name>fs.default.name</name>
      <value>hdfs://localhost:60908</value>
     </property>
    <property>
      <name>hadoop.security.key.provider.path</name>
      <value>kms://http@localhost:16000/kms</value>
    </property>
      <property>
        <name>dfs.encryption.key.provider.uri</name>
        <value>kms://http@localhost:16000/kms</value>
     </property>
    </configuration>
    
In the hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/mini-conf direcotry, create a file called hdfs-site.xml with 
the following content:

    <configuration>
      <property>
        <name>hadoop.security.key.provider.path</name>
        <value>kms://http@localhost:16000/kms</value>
      </property>
      <property>
       <name>dfs.encryption.key.provider.uri</name>
       <value>kms://http@localhost:16000/kms</value>
      </property>
    </configuration>

If your name node port is not 60908, replace 60908 above with the appropriate value.

#### Create an HDFS directory tree and encryption zone with content

(preferably in a separate shell from the KMS process and Hadoop mini-cluster process above)

    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/
    $ bin/hadoop fs -fs hdfs://localhost:60908 -ls /tmp
    $ bin/hadoop fs -fs hdfs://localhost:60908 -mkdir /tmp/piano
    $ bin/hadoop key create pianokey
    $ bin/hdfs --config mini-conf/ crypto -createZone -keyName pianokey -path /tmp/piano
    $ echo "This is a test." > readme.txt
    $ bin/hadoop fs -fs hdfs://localhost:60908 -put readme.txt /tmp/piano
    $ bin/hadoop fs -fs hdfs://localhost:60908 -cat /tmp/piano/readme.txt
    
#### Stop KMS

    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/
    $ .sbin/kms.sh stop
    
#### Configure KMS to use TKP in migration mode

(set the following value in hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/etc/hadoop)

      <property>
        <name>hadoop.kms.key.provider.uri</name>
        <value>keytrustee://file@/${user.home}/.keytrustee/,jceks://file@/${user.home}/kms.keystore</value>
        <!--<value>jceks://file@/${user.home}/kms.keystore</value>-->
        <description>
          URI of the backing KeyProvider for the KMS.
        </description>
      </property>

#### Start KMS

    $ cd hadoop-dist/target/hadoop-3.0.0-SNAPSHOT/
    $ .sbin/kms.sh start
    
#### Verify that you cannot read encrypted content now

    $ bin/hadoop fs -fs hdfs://localhost:60908 -cat /tmp/piano/readme.txt
    
#### Migrate keys

    $ curl L -d "trusteeOp=migrate" "http://localhost:16000/kms/v1/trustee/key/migrate?user.name=anthony&trusteeOp=migrate"

    (You can monitor logs/kms-log.log and logs/kms-catalina.out to verify KMS startup and migration are successful)
        
#### Verify that you can read encrypted content now

    $ bin/hadoop fs -fs hdfs://localhost:60908 -cat /tmp/piano/readme.txt

EXTRA STUFF
-----------
When building debian package, DEBEMAIL and DEBFULLNAME env variables will override Maintainer value in control file.

To start KMS in debug mode: kms.sh jpda start
To set KMS to wait for debugger attach before starting Tomcat set environment variable JPDA_SUSPEND=y 
before running command above.

To start Hadoop mini-cluster in debug configurations see 
https://wiki.cloudera.com/display/~yzhang/Debug+Hadoop+Components+Using+Eclipse

To run curl when Kerberos is enabled follow the instructions at http://www.cloudera.com/content/cloudera/en/documentation/core/latest/topics/cdh_sg_httpfs_security.html?scroll=topic_7_2_unique_7
